@@ -11,6 +11,7 @@ get.e0.mcmc <- function(sim.dir = file.path(getwd(), 'bayesLife.output'),
 	}
 	load(file = mcmc.file.path)
 	bayesLife.mcmc.meta$output.dir <- normalizePath(sim.dir)
+	if(is.null(bayesLife.mcmc.meta$annual.simulation)) bayesLife.mcmc.meta$annual.simulation <- FALSE
 	if(is.null(bayesLife.mcmc.meta$mcmc.options)) {
 	    bayesLife.mcmc.meta <- .convert.meta.from.legacy.form(bayesLife.mcmc.meta)
 	}
@@ -136,6 +137,34 @@ get.e0.prediction <- function(mcmc=NULL, sim.dir=NULL, joint.male=FALSE, mcmc.di
 	return(pred)
 }
 
+get.e0.jmale.prediction <- function(e0.pred) {
+    male.pred <- e0.pred$joint.male
+    if(is.null(male.pred)) stop('A joint male prediction does not exist for the given object. Use e0.jmale.predict to simulate male projections from existing female projections.')
+    male.pred$mcmc.set <- e0.pred$mcmc.set
+    for(item in names(male.pred$meta.changes))
+        male.pred$mcmc.set$meta[[item]] <- male.pred$meta.changes[[item]]
+    return(male.pred)
+}
+
+has.e0.jmale.prediction <- function(e0.pred) return(!is.null(e0.pred$joint.male))
+
+get.rege0.prediction <- function(sim.dir, country = NULL, method = "ar1", joint.male = FALSE){
+    dir <- file.path(sim.dir, paste0('subnat_', method))
+    if(!file.exists(dir)) stop("No subnational predictions in ", sim.dir)
+    cdirs <- list.files(dir, pattern='^c[0-9]+', full.names=FALSE)
+    if(length(cdirs)==0) stop("No subnational predictions in ", sim.dir)
+    if (!is.null(country)) {
+        cdirs <- cdirs[cdirs == paste0("c", country)]
+        if(length(cdirs)==0) stop("No subnational predictions for country ", country, " in ", sim.dir)
+        return(get.e0.prediction(sim.dir=file.path(dir, cdirs[1]), mcmc.dir=NA, joint.male = joint.male))
+    }
+    preds <- NULL
+    for (d in cdirs) 
+        preds[[sub("c", "", d)]] <- get.e0.prediction(sim.dir=file.path(dir, d), mcmc.dir=NA, joint.male = joint.male)
+    return(preds)
+}
+
+
 get.e0.convergence.all <- function(sim.dir=file.path(getwd(), 'bayesLife.output')) {
 	return(bayesTFR:::.do.get.convergence.all('e0', 'bayesLife', sim.dir=sim.dir))
 }
@@ -268,11 +297,20 @@ get.mcmc.meta.bayesLife.prediction <- function(meta, ...) return(meta$mcmc.set$m
 
 
 summary.bayesLife.mcmc <- function(object, country = NULL, 
-								par.names = NULL, par.names.cs = NULL, ...) {
+								par.names = NULL, par.names.cs = NULL, thin = 1, burnin = 0, ...) {
+    res <- list()
+    class(res) <- "summary.bayesTFR.mcmc"
     if(missing(par.names)) par.names <- e0.parameter.names(get.mcmc.meta(object)$mcmc.options)
     if(missing(par.names.cs)) par.names.cs <- e0.parameter.names.cs(get.mcmc.meta(object)$mcmc.options)
-	return (bayesTFR:::summary.bayesTFR.mcmc(object, country = country,
-				par.names = par.names, par.names.cs = par.names.cs, ...))
+    if (!is.null(country)) {
+        country.obj <- get.country.object(country, object$meta)
+        if(is.null(country.obj$name)) stop("Country ", country, " not found.")
+        res$country.name <- country.obj$name
+        country <- country.obj$code
+    } 
+    res$results <- summary(coda.mcmc(object, country=country, par.names=par.names,
+                                     par.names.cs=par.names.cs, thin=thin, burnin=burnin), ...)
+	return(res)
 }
 
 summary.bayesLife.mcmc.set <- function(object, country=NULL, chain.id=NULL, 
@@ -282,39 +320,90 @@ summary.bayesLife.mcmc.set <- function(object, country=NULL, chain.id=NULL,
     if(missing(par.names)) par.names <- e0.parameter.names(get.mcmc.meta(object)$mcmc.options)
     if(missing(par.names.cs) && !is.null(country)) par.names.cs <- e0.parameter.names.cs(get.mcmc.meta(object)$mcmc.options)
 
-	cat('\nSimulation:', get.sex.label(object$meta), 'life expectancy')
-	cat('\nWPP:', object$meta$wpp.year)
-	cat('\nInput data: e0 for period', object$meta$start.year, '-', object$meta$present.year,'\n')
-	
-	cat('\nMCMC parameters estimated for', object$meta$nr.countries, 'countries.')
-	cat('\nHyperparameters estimated using', object$meta$nr.countries.estimation, 'countries.')
-	cat('\n')
+	res <- list(meta = summary(object$meta))
+	class(res) <- "summary.bayesLife.mcmc.set"
 	if(meta.only) {
-		get.iter <- function(x) x$finished.iter
-		res <- list(nr.chains=object$meta$nr.chains, 
-					iters=sapply(object$mcmc.list, get.iter),
-					thin=object$mcmc.list[[1]]$thin)
-		class(res) <- 'summary.bayesLife.mcmc.set.meta'
-
-		return(res)
-	} 
-	if (!is.null(chain.id))
-		return(summary(object$mcmc.list[[chain.id]], country = country, 
-		               par.names = par.names, par.names.cs = par.names.cs, 
-		               thin = thin, burnin = burnin, ...))
-	if (!is.null(country)) {
-		country.obj <- get.country.object(country, object$meta)
-		if(is.null(country.obj$name)) stop("Country ", country, " not found.")
-		cat('\nCountry:', country.obj$name, '\n')
-		country <- country.obj$code
+	    res$chain.info <- bayesTFR:::chain.info(object)
+	    return(res)
 	}
-	summary(coda.list.mcmc(object, country = country, par.names = par.names,
+	if (!is.null(chain.id)) {
+	    res$mcmc <- summary(object$mcmc.list[[chain.id]], country = country, 
+	                        par.names = par.names, par.names.cs = par.names.cs, 
+	                        thin = thin, burnin = burnin, ...)
+        return(res)
+	}
+	if (!is.null(country)) {
+	    country.obj <- get.country.object(country, object$meta)
+	    if(is.null(country.obj$name)) stop("Country ", country, " not found.")
+	    res$country.name <- country.obj$name
+	    country <- country.obj$code
+	}
+	res$results <- summary(coda.list.mcmc(object, country = country, par.names = par.names,
 							par.names.cs = par.names.cs, thin = thin, 
 							burnin = burnin), ...)
+	return(res)
 }
 
-print.summary.bayesLife.mcmc.set.meta <- function(x, ...) 
-    return(bayesTFR:::print.summary.bayesTFR.mcmc.set.meta(x, ...))
+summary.bayesLife.mcmc.meta <- function(object, ...) {
+    res <- list(sex.label = get.sex.label(object),
+                est.period = paste(object$start.year, object$present.year, sep = '-'),
+                nr.countries = object$nr.countries,
+                nr.countries.est = object$nr.countries.estimation,
+                wpp.year = object$wpp.year,
+                annual = if(!is.null(object$annual.simulation)) object$annual.simulation else FALSE
+                )
+    class(res) <- "summary.bayesLife.mcmc.meta"
+    return(res)
+}
+
+print.summary.bayesLife.mcmc.meta <- function(x, ...) {
+    cat('\nSimulation:', x$sex.label, 'life expectancy')
+    cat('\nNumber of countries:', x$nr.countries)
+    cat('\nHyperparameters estimated using', x$nr.countries.est, 'countries.')
+    cat('\nWPP:', x$wpp.year)
+    cat('\nInput data: e0 for period', x$est.period)
+    cat('\nTime interval: ')
+    if(x$annual) cat('annual') else cat('5-year') 
+    cat('\n')
+}
+
+print.summary.bayesLife.mcmc.set <- function(x, ...) {
+    print(x$meta)
+    if(!is.null(x$chain.info)) bayesTFR:::print.summary.chain.info(x$chain.info)
+    if(!is.null(x$mcmc)) print(x$mcmc)
+    if(!is.null(x$country.name)){
+        cat('\nCountry:', x$country.name, '\n')
+        if (is.null(x$results))
+            cat('\tnot used for estimation.\n')
+    }
+    if(!is.null(x$results)) print(x$results)
+}
+
+print.bayesLife.mcmc <- function(x, ...) {
+    print(summary(x, ...))
+}
+
+print.summary.bayesLife.mcmc <- function(x, ...) {
+    if(!is.null(x$country.name)){
+        cat('\nCountry:', x$country.name, '\n')
+        if (is.null(x$results))
+            cat('\tnot used for estimation.\n')
+    }
+    if(!is.null(x$results))
+        print(x$results)
+}
+
+print.bayesLife.mcmc.set <- function(x, ...) {
+    print(summary(x, ...))
+}
+
+print.bayesLife.mcmc.meta <- function(x, ...) {
+    print(summary(x, ...))
+}
+
+print.bayesLife.prediction <- function(x, ...) {
+    print(summary(x, ...))
+}
 
 summary.bayesLife.prediction <- function(object, country = NULL, compact = TRUE, ...) {
 	res <- bayesTFR:::get.prediction.summary.data(object, 
@@ -345,6 +434,9 @@ print.summary.bayesLife.prediction <- function(x, digits = 3, ...) {
 	}
 }
 
+
+
+
 summary.bayesLife.convergence <- function(object, expand=FALSE, ...) {
 	return(bayesTFR:::summary.bayesTFR.convergence(object, expand=expand, ...))
 }
